@@ -4,5 +4,6 @@ well as utilities to help in plotting.
 """
 
 from .pyqt_plotmon import PlotMonitor_pyqt
+from .instrument_monitor import InstrumentMonitor
 
-__all__ = ['PlotMonitor_pyqt']
+__all__ = ['PlotMonitor_pyqt', 'InstrumentMonitor']
