@@ -8,18 +8,21 @@ test_datadir = get_test_data_dir()
 
 
 def test_create_plotmon_from_historical():
+    # Always set datadir before instruments
     set_datadir(test_datadir)
-    plotmon = create_plotmon_from_historical('20200504-191556-002-4209ee')
+    tuid = '20200504-191556-002-4209ee'
+    plotmon = create_plotmon_from_historical(tuid)
 
-    x = plotmon.curves[0]['config']['x']
-    y = plotmon.curves[0]['config']['y']
+    curves_dict = plotmon._get_curves_config()
+    x = curves_dict[tuid]["x0y0"]['config']['x']
+    y = curves_dict[tuid]["x0y0"]['config']['y']
 
     x_exp = np.linspace(0, 5, 50)
     y_exp = np.cos(np.pi * x_exp) * -1
     np.testing.assert_allclose(x[:50], x_exp)
     np.testing.assert_allclose(y[:50], y_exp)
 
-    cfg = plotmon.secondary_QtPlot.traces[0]['config']
+    cfg = plotmon._get_traces_config(which="secondary_QtPlot")[0]["config"]
     assert np.shape(cfg['z']) == (11, 50)
     assert cfg['xlabel'] == 'Time'
     assert cfg['xunit'] == 's'
